@@ -50,12 +50,12 @@ typedef enum {
 static void nm_device_wifi_refresh_ui (NetDeviceWifi *device_wifi);
 static void show_wifi_list (NetDeviceWifi *device_wifi);
 static void populate_ap_list (NetDeviceWifi *device_wifi);
+static void show_hotspot_ui (NetDeviceWifi *device_wifi);
 
 struct _NetDeviceWifiPrivate
 {
         GtkBuilder              *builder;
         GtkWidget               *details_dialog;
-        GtkWidget               *hotspot_dialog;
         GtkSwitch               *hotspot_switch;
         gboolean                 updating_device;
         gchar                   *selected_ssid_title;
@@ -325,9 +325,8 @@ device_get_hotspot_ssid (NetDeviceWifi *device_wifi,
         NMSettingWireless *sw;
 
         c = find_connection_for_device (device_wifi, device);
-        if (c == NULL) {
-                return FALSE;
-        }
+        if (c == NULL)
+                return NULL;
 
         sw = nm_connection_get_setting_wireless (c);
         return nm_setting_wireless_get_ssid (sw);
@@ -344,8 +343,9 @@ get_secrets_cb (GObject            *source_object,
 
         secrets = nm_remote_connection_get_secrets_finish (NM_REMOTE_CONNECTION (source_object), res, &error);
         if (!secrets) {
+                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+                        g_warning ("Could not get secrets: %s", error->message);
                 g_error_free (error);
-                //FIXME ignore cancelled
                 return;
         }
 
@@ -434,6 +434,9 @@ nm_device_wifi_refresh_hotspot (NetDeviceWifi *device_wifi)
                                              &hotspot_secret,
                                              &hotspot_security);
 
+        g_debug ("Refreshing hotspot labels to name: '%s', security key: '%s', security: '%s'",
+                 hotspot_ssid, hotspot_secret, hotspot_security);
+
         panel_set_device_widget_details (device_wifi->priv->builder,
                                          "hotspot_network_name",
                                          hotspot_ssid);
@@ -498,7 +501,6 @@ static void
 nm_device_wifi_refresh_ui (NetDeviceWifi *device_wifi)
 {
         const gchar *str;
-        gboolean is_hotspot;
         gchar *str_tmp = NULL;
         gint strength = 0;
         guint speed = 0;
@@ -511,9 +513,9 @@ nm_device_wifi_refresh_ui (NetDeviceWifi *device_wifi)
         NetDeviceWifiPrivate *priv = device_wifi->priv;
         GtkWidget *dialog;
 
-        is_hotspot = device_is_hotspot (device_wifi);
-        if (is_hotspot) {
+        if (device_is_hotspot (device_wifi)) {
                 nm_device_wifi_refresh_hotspot (device_wifi);
+                show_hotspot_ui (device_wifi);
                 return;
         }
 
@@ -602,6 +604,7 @@ nm_device_wifi_refresh_ui (NetDeviceWifi *device_wifi)
         panel_set_device_status (priv->builder, "heading_status", nm_device, NULL);
 
         /* update list of APs */
+        show_wifi_list (device_wifi);
         populate_ap_list (device_wifi);
 }
 
@@ -651,12 +654,13 @@ connection_add_activate_cb (GObject *source_object,
 
         conn = nm_client_add_and_activate_connection_finish (NM_CLIENT (source_object), res, &error);
         if (!conn) {
-                //FIXME cancelled
-                nm_device_wifi_refresh_ui (user_data);
-                /* failed to activate */
-                g_debug ("Failed to add and activate connection '%d': %s",
-                         error->code,
-                         error->message);
+                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+                        nm_device_wifi_refresh_ui (user_data);
+                        /* failed to activate */
+                        g_warning ("Failed to add and activate connection '%d': %s",
+                                   error->code,
+                                   error->message);
+                }
                 g_error_free (error);
                 return;
         }
@@ -670,12 +674,13 @@ connection_activate_cb (GObject *source_object,
         GError *error = NULL;
 
         if (!nm_client_activate_connection_finish (NM_CLIENT (source_object), res, &error)) {
-                //FIXME cancelled
-                nm_device_wifi_refresh_ui (user_data);
-                /* failed to activate */
-                g_debug ("Failed to add and activate connection '%d': %s",
-                         error->code,
-                         error->message);
+                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+                        nm_device_wifi_refresh_ui (user_data);
+                        /* failed to activate */
+                        g_debug ("Failed to add and activate connection '%d': %s",
+                                 error->code,
+                                 error->message);
+                }
                 g_error_free (error);
                 return;
         }
@@ -995,7 +1000,6 @@ activate_cb (GObject            *source_object,
 
         /* show hotspot tab */
         nm_device_wifi_refresh_ui (user_data);
-        show_hotspot_ui (user_data);
 }
 
 static void
@@ -1018,14 +1022,68 @@ activate_new_cb (GObject            *source_object,
 
         /* show hotspot tab */
         nm_device_wifi_refresh_ui (user_data);
-        show_hotspot_ui (user_data);
+}
+
+static NMConnection *
+net_device_wifi_get_hotspot_connection (NetDeviceWifi *device_wifi)
+{
+        GSList *connections, *l;
+        NMConnection *c = NULL;
+
+        connections = net_device_get_valid_connections (NET_DEVICE (device_wifi));
+        for (l = connections; l; l = l->next) {
+                NMConnection *tmp = l->data;
+                if (is_hotspot_connection (tmp)) {
+                        c = tmp;
+                        break;
+                }
+        }
+        g_slist_free (connections);
+
+        return c;
+}
+
+static void
+overwrite_ssid_cb (GObject      *source_object,
+                   GAsyncResult *res,
+                   gpointer      user_data)
+{
+        GError *error = NULL;
+        NMClient *client;
+        NMRemoteConnection *connection;
+        NMDevice *device;
+        NMConnection *c;
+        NetDeviceWifi *device_wifi;
+
+        connection = NM_REMOTE_CONNECTION (source_object);
+
+        if (!nm_remote_connection_commit_changes_finish (connection, res, &error)) {
+                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+                        g_warning ("Failed to save hotspot's settings to disk: %s",
+                                   error->message);
+                g_error_free (error);
+                return;
+        }
+
+        device_wifi = user_data;
+        device = net_device_get_nm_device (NET_DEVICE (device_wifi));
+        client = net_object_get_client (NET_OBJECT (device_wifi));
+        c = net_device_wifi_get_hotspot_connection (device_wifi);
+
+        g_debug ("activate existing hotspot connection\n");
+        nm_client_activate_connection_async (client,
+                                             c,
+                                             device,
+                                             NULL,
+                                             NULL,
+                                             activate_cb,
+                                             device_wifi);
 }
 
 static void
 start_shared_connection (NetDeviceWifi *device_wifi)
 {
         NMConnection *c;
-        NMConnection *tmp;
         NMSettingConnection *sc;
         NMSettingWireless *sw;
         NMSettingIP4Config *sip;
@@ -1034,8 +1092,6 @@ start_shared_connection (NetDeviceWifi *device_wifi)
         GBytes *ssid;
         const gchar *str_mac;
         struct ether_addr *bin_mac;
-        GSList *connections;
-        GSList *l;
         NMClient *client;
         const char *mode;
         NMDeviceWifiCapabilities caps;
@@ -1043,39 +1099,35 @@ start_shared_connection (NetDeviceWifi *device_wifi)
         device = net_device_get_nm_device (NET_DEVICE (device_wifi));
         g_assert (nm_device_get_device_type (device) == NM_DEVICE_TYPE_WIFI);
 
-        connections = net_device_get_valid_connections (NET_DEVICE (device_wifi));
-        c = NULL;
-        for (l = connections; l; l = l->next) {
-                tmp = l->data;
-                if (is_hotspot_connection (tmp)) {
-                        c = tmp;
-                        break;
-                }
-        }
-        g_slist_free (connections);
+        c = net_device_wifi_get_hotspot_connection (device_wifi);
 
         ssid = generate_ssid_for_hotspot (device_wifi);
 
         client = net_object_get_client (NET_OBJECT (device_wifi));
         if (c != NULL) {
                 NMSettingWireless *sw;
+                const char *c_path;
+                NMRemoteConnection *connection;
 
                 sw = nm_connection_get_setting_wireless (c);
                 g_object_set (sw, "ssid", ssid, NULL);
                 g_bytes_unref (ssid);
 
-                g_debug ("activate existing hotspot connection\n");
-                nm_client_activate_connection_async (client,
-                                                     c,
-                                                     device,
-                                                     NULL,
-                                                     NULL,
-                                                     activate_cb,
-                                                     device_wifi);
+                c_path = nm_connection_get_path (c);
+                connection = nm_client_get_connection_by_path (client, c_path);
+
+                g_debug ("overwriting ssid to %s", (char *) g_bytes_get_data (ssid, NULL));
+
+                nm_remote_connection_commit_changes_async (connection,
+                                                           TRUE,
+                                                           NULL,
+                                                           overwrite_ssid_cb,
+                                                           device_wifi);
                 return;
         }
 
-        g_debug ("create new hotspot connection\n");
+        g_debug ("create new hotspot connection with SSID '%s'",
+                 (char *) g_bytes_get_data (ssid, NULL));
         c = nm_simple_connection_new ();
 
         sc = (NMSettingConnection *)nm_setting_connection_new ();
@@ -1159,7 +1211,7 @@ start_hotspot_response_cb (GtkWidget *dialog, gint response, NetDeviceWifi *devi
         if (response == GTK_RESPONSE_OK) {
                 start_shared_connection (device_wifi);
         }
-        gtk_widget_hide (dialog);
+        gtk_widget_destroy (dialog);
 }
 
 static void
@@ -1171,7 +1223,8 @@ start_hotspot (GtkButton *button, NetDeviceWifi *device_wifi)
         NMClient *client;
         GtkWidget *dialog;
         GtkWidget *window;
-        GtkWidget *widget;
+        GtkWidget *message_area;
+        GtkWidget *label;
         GString *str;
 
         active_ssid = NULL;
@@ -1199,11 +1252,7 @@ start_hotspot (GtkButton *button, NetDeviceWifi *device_wifi)
 
         window = gtk_widget_get_toplevel (GTK_WIDGET (button));
 
-        dialog = device_wifi->priv->hotspot_dialog;
-        gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
-
-        str = g_string_new (_("If you have a connection to the Internet other than wireless, you can set up a wireless hotspot to share the connection with others."));
-        g_string_append (str, "\n\n");
+        str = g_string_new ("");
 
         if (active_ssid) {
                 g_string_append_printf (str, _("Switching on the wireless hotspot will disconnect you from <b>%s</b>."), active_ssid);
@@ -1212,17 +1261,52 @@ start_hotspot (GtkButton *button, NetDeviceWifi *device_wifi)
 
         g_string_append (str, _("It is not possible to access the Internet through your wireless while the hotspot is active."));
 
-        widget = GTK_WIDGET (gtk_builder_get_object (device_wifi->priv->builder, "hotspot-dialog-content"));
-        gtk_label_set_markup (GTK_LABEL (widget), str->str);
-        g_string_free (str, TRUE);
+        dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (window),
+                                                     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                     GTK_MESSAGE_OTHER,
+                                                     GTK_BUTTONS_NONE,
+                                                     "<big><b>%s</b></big>",
+                                                     _("Turn On Wi-Fi Hotspot?"));
 
+        /* Because we can't control the text alignment with markup, add
+         * the strings as labels directly */
+        message_area = gtk_message_dialog_get_message_area (GTK_MESSAGE_DIALOG (dialog));
+
+        label = g_object_new (GTK_TYPE_LABEL,
+                              "use-markup", TRUE,
+                              "xalign", 0.5,
+                              "max-width-chars", 50,
+                              "wrap", TRUE,
+                              "label", str->str,
+                              "justify", GTK_JUSTIFY_CENTER,
+                              "halign", GTK_ALIGN_CENTER,
+                              NULL);
+        gtk_container_add (GTK_CONTAINER (message_area), label);
+
+        label = g_object_new (GTK_TYPE_LABEL,
+                              "use-markup", TRUE,
+                              "xalign", 0.5,
+                              "max-width-chars", 50,
+                              "wrap", TRUE,
+                              "label", _("Wi-Fi hotspots are usually used to share an additional Internet connection over Wi-Fi."),
+                              "justify", GTK_JUSTIFY_CENTER,
+                              "halign", GTK_ALIGN_CENTER,
+                              NULL);
+        gtk_style_context_add_class (gtk_widget_get_style_context (label), "dim-label");
+        gtk_container_add (GTK_CONTAINER (message_area), label);
+
+        gtk_widget_show_all (message_area);
+
+        gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+                                _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                _("_Turn On"), GTK_RESPONSE_OK,
+                                NULL);
         g_signal_connect (dialog, "response",
                           G_CALLBACK (start_hotspot_response_cb), device_wifi);
-        g_signal_connect (dialog, "delete-event",
-                        G_CALLBACK (gtk_widget_hide_on_delete), NULL);
 
         gtk_window_present (GTK_WINDOW (dialog));
         g_free (active_ssid);
+        g_string_free (str, TRUE);
 }
 
 static void
@@ -1259,7 +1343,6 @@ stop_shared_connection (NetDeviceWifi *device_wifi)
         }
 
         nm_device_wifi_refresh_ui (device_wifi);
-        show_wifi_list (device_wifi);
 }
 
 static void
@@ -1318,14 +1401,16 @@ client_connection_added_cb (NMClient           *client,
 {
         gboolean is_hotspot;
 
-        populate_ap_list (device_wifi);
-
         /* go straight to the hotspot UI */
         is_hotspot = device_is_hotspot (device_wifi);
         if (is_hotspot) {
                 nm_device_wifi_refresh_hotspot (device_wifi);
                 show_hotspot_ui (device_wifi);
+                return;
         }
+
+        populate_ap_list (device_wifi);
+        show_wifi_list (device_wifi);
 }
 
 static void
@@ -1420,7 +1505,6 @@ net_device_wifi_finalize (GObject *object)
         NetDeviceWifiPrivate *priv = device_wifi->priv;
 
         g_clear_pointer (&priv->details_dialog, gtk_widget_destroy);
-        g_clear_pointer (&priv->hotspot_dialog, gtk_widget_destroy);
         g_object_unref (priv->builder);
         g_free (priv->selected_ssid_title);
         g_free (priv->selected_connection_id);
@@ -1822,7 +1906,10 @@ open_history (NetDeviceWifi *device_wifi)
         CcNetworkPanel *panel;
         GtkWidget *button;
         GtkWidget *forget;
+        GtkWidget *header;
         GtkWidget *swin;
+        GtkWidget *content_area;
+        GtkWidget *separator;
         GSList *connections;
         GSList *l;
         const GPtrArray *aps;
@@ -1835,40 +1922,36 @@ open_history (NetDeviceWifi *device_wifi)
         GtkSizeGroup *rows;
         GtkSizeGroup *icons;
 
-        dialog = gtk_dialog_new ();
+        dialog = g_object_new (GTK_TYPE_DIALOG, "use-header-bar", 1, NULL);
         panel = net_object_get_panel (NET_OBJECT (device_wifi));
         window = gtk_widget_get_toplevel (GTK_WIDGET (panel));
         gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
-        gtk_window_set_title (GTK_WINDOW (dialog), _("History"));
+        gtk_window_set_title (GTK_WINDOW (dialog), _("Known Wi-Fi Networks"));
         gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-        gtk_window_set_default_size (GTK_WINDOW (dialog), 600, 400);
+        gtk_window_set_default_size (GTK_WINDOW (dialog), 500, 400);
 
-        button = gtk_button_new_with_mnemonic (_("_Close"));
-        gtk_widget_set_can_default (button, TRUE);
-        gtk_widget_show (button);
-        gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, 0);
-        g_signal_connect_swapped (button, "clicked",
+        /* Dialog's header */
+        header = gtk_header_bar_new ();
+        gtk_widget_show (header);
+        gtk_header_bar_set_title (GTK_HEADER_BAR (header), _("Known Wi-Fi Networks"));
+        gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (header), TRUE);
+        gtk_window_set_titlebar (GTK_WINDOW (dialog), header);
+
+        g_signal_connect_swapped (dialog, "response",
                                   G_CALLBACK (gtk_widget_destroy), dialog);
 
-        /* translators: This is the label for the "Forget wireless network" functionality */
-        forget = gtk_button_new_with_mnemonic (C_("Wi-Fi Network", "_Forget"));
-        gtk_widget_show (forget);
-        gtk_widget_set_sensitive (forget, FALSE);
-        gtk_dialog_add_action_widget (GTK_DIALOG (dialog), forget, 0);
-        g_signal_connect (forget, "clicked",
-                          G_CALLBACK (forget_selected), device_wifi);
-        gtk_container_child_set (GTK_CONTAINER (gtk_widget_get_parent (forget)), forget, "secondary", TRUE, NULL);
-        g_object_set_data (G_OBJECT (forget), "net", device_wifi);
+        g_signal_connect_swapped (dialog, "delete-event",
+                                  G_CALLBACK (gtk_widget_destroy), dialog);
+
+        content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+        gtk_container_set_border_width (GTK_CONTAINER (content_area), 0);
 
         swin = gtk_scrolled_window_new (NULL, NULL);
         gtk_widget_show (swin);
+        gtk_widget_set_vexpand (swin, TRUE);
         gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-        gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (swin), GTK_SHADOW_IN);
-        gtk_widget_set_margin_start (swin, 50);
-        gtk_widget_set_margin_end (swin, 50);
-        gtk_widget_set_margin_top (swin, 12);
-        gtk_widget_set_margin_bottom (swin, 12);
-        gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), swin, TRUE, TRUE, 0);
+        gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (swin), GTK_SHADOW_NONE);
+        gtk_container_add (GTK_CONTAINER (content_area), swin);
 
         list = GTK_WIDGET (gtk_list_box_new ());
         gtk_widget_show (list);
@@ -1877,6 +1960,26 @@ open_history (NetDeviceWifi *device_wifi)
         gtk_list_box_set_sort_func (GTK_LIST_BOX (list), (GtkListBoxSortFunc)history_sort, NULL, NULL);
         gtk_container_add (GTK_CONTAINER (swin), list);
 
+        /* Horizontal separator */
+        separator = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
+        gtk_widget_show (separator);
+        gtk_container_add (GTK_CONTAINER (content_area), separator);
+
+        /* translators: This is the label for the "Forget wireless network" functionality */
+        forget = gtk_button_new_with_mnemonic (C_("Wi-Fi Network", "_Forget"));
+        gtk_widget_show (forget);
+        gtk_widget_set_halign (forget, GTK_ALIGN_START);
+        gtk_widget_set_margin_top (forget, 6);
+        gtk_widget_set_margin_bottom (forget, 6);
+        gtk_widget_set_margin_start (forget, 6);
+        gtk_widget_set_sensitive (forget, FALSE);
+        gtk_style_context_add_class (gtk_widget_get_style_context (forget), "destructive-action");
+
+        g_signal_connect (forget, "clicked",
+                          G_CALLBACK (forget_selected), device_wifi);
+        g_object_set_data (G_OBJECT (forget), "net", device_wifi);
+        gtk_container_add (GTK_CONTAINER (content_area), forget);
+
         rows = gtk_size_group_new (GTK_SIZE_GROUP_VERTICAL);
         icons = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
         g_object_set_data_full (G_OBJECT (list), "rows", rows, g_object_unref);
@@ -2066,10 +2169,6 @@ net_device_wifi_init (NetDeviceWifi *device_wifi)
                                                      "details_dialog"));
         device_wifi->priv->details_dialog = widget;
 
-        widget = GTK_WIDGET (gtk_builder_get_object (device_wifi->priv->builder,
-                                                     "hotspot-dialog"));
-        device_wifi->priv->hotspot_dialog = widget;
-
         /* setup wifi views */
         widget = GTK_WIDGET (gtk_builder_get_object (device_wifi->priv->builder,
                                                      "device_off_switch"));
