@@ -36,17 +36,20 @@
 #include "cc-editable-entry.h"
 #include "pp-details-dialog.h"
 #include "pp-ppd-selection-dialog.h"
+#include "pp-printer.h"
 #include "pp-utils.h"
 
 struct _PpDetailsDialog {
   GtkDialog parent;
 
+  GtkLabel *dialog_title;
   GtkEntry *printer_name_entry;
   GtkEntry *printer_location_entry;
   GtkLabel *printer_address_label;
   GtkLabel *printer_model_label;
   GtkStack *printer_model_stack;
   GtkWidget *search_for_drivers_button;
+  GtkWidget *driver_buttons;
 
   gchar        *printer_name;
   gchar        *printer_location;
@@ -64,24 +67,46 @@ struct _PpDetailsDialogClass
   GtkDialogClass parent_class;
 };
 
-G_DEFINE_TYPE (PpDetailsDialog, pp_details_dialog, GTK_TYPE_DIALOG);
+G_DEFINE_TYPE (PpDetailsDialog, pp_details_dialog, GTK_TYPE_DIALOG)
 
-static gboolean
-printer_name_edit_cb (GtkWidget       *entry,
-                      GdkEventFocus   *event,
-                      PpDetailsDialog *self)
+static void
+on_printer_rename_cb (GObject      *source_object,
+                      GAsyncResult *result,
+                      gpointer      user_data)
 {
+  pp_printer_rename_finish (PP_PRINTER (source_object), result, NULL);
+
+  g_object_unref (source_object);
+}
+
+static void
+pp_details_dialog_response_cb (GtkDialog *dialog,
+                               gint       response_id,
+                               gpointer   user_data)
+{
+  PpDetailsDialog *self = (PpDetailsDialog*) dialog;
   const gchar *new_name;
+  const gchar *new_location;
+
+  new_location = gtk_entry_get_text (GTK_ENTRY (self->printer_location_entry));
+  if (g_strcmp0 (self->printer_location, new_location) != 0)
+    {
+      printer_set_location (self->printer_name, new_location);
+
+      self->printer_location = g_strdup (new_location);
+    }
 
   new_name = gtk_entry_get_text (GTK_ENTRY (self->printer_name_entry));
   if (g_strcmp0 (self->printer_name, new_name) != 0)
     {
-      printer_rename (self->printer_name, new_name);
+      PpPrinter *printer = pp_printer_new (self->printer_name);
 
-      self->printer_name = g_strdup (new_name);
+      pp_printer_rename_async (printer,
+                               new_name,
+                               NULL,
+                               on_printer_rename_cb,
+                               NULL);
     }
-
-  return FALSE;
 }
 
 static void
@@ -89,7 +114,6 @@ printer_name_changed (GtkEditable *editable,
                       gpointer     user_data)
 {
   PpDetailsDialog *self = (PpDetailsDialog *) user_data;
-  GtkWidget *widget;
   const gchar *name;
   gchar *title;
 
@@ -97,31 +121,11 @@ printer_name_changed (GtkEditable *editable,
 
   /* Translators: This is the title of the dialog. %s is the printer name. */
   title = g_strdup_printf (_("%s Details"), name);
-
-  widget = gtk_dialog_get_header_bar (GTK_DIALOG (self));
-  gtk_header_bar_set_title (GTK_HEADER_BAR (widget), title);
+  gtk_label_set_label (self->dialog_title, title);
 
   g_free (title);
 }
 
-static gboolean
-printer_location_edit_cb (GtkWidget       *entry,
-                          GdkEventFocus   *event,
-                          PpDetailsDialog *self)
-{
-  const gchar *location;
-
-  location = gtk_entry_get_text (GTK_ENTRY (self->printer_location_entry));
-  if (g_strcmp0 (self->printer_location, location) != 0)
-    {
-      printer_set_location (self->printer_name, location);
-
-      self->printer_location = g_strdup (location);
-    }
-
-  return FALSE;
-}
-
 static void
 ppd_names_free (gpointer user_data)
 {
@@ -348,6 +352,15 @@ select_ppd_manually (GtkButton       *button,
   gtk_widget_destroy (dialog);
 }
 
+static void
+update_sensitivity (PpDetailsDialog *self,
+                    gboolean         sensitive)
+{
+  gtk_widget_set_sensitive (GTK_WIDGET (self->printer_name_entry), sensitive);
+  gtk_widget_set_sensitive (GTK_WIDGET (self->printer_location_entry), sensitive);
+  gtk_widget_set_sensitive (GTK_WIDGET (self->driver_buttons), sensitive);
+}
+
 static void
 pp_details_dialog_init (PpDetailsDialog *self)
 {
@@ -361,19 +374,20 @@ pp_details_dialog_class_init (PpDetailsDialogClass *klass)
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/printers/details-dialog.ui");
 
+  gtk_widget_class_bind_template_child (widget_class, PpDetailsDialog, dialog_title);
   gtk_widget_class_bind_template_child (widget_class, PpDetailsDialog, printer_name_entry);
   gtk_widget_class_bind_template_child (widget_class, PpDetailsDialog, printer_location_entry);
   gtk_widget_class_bind_template_child (widget_class, PpDetailsDialog, printer_address_label);
   gtk_widget_class_bind_template_child (widget_class, PpDetailsDialog, printer_model_label);
   gtk_widget_class_bind_template_child (widget_class, PpDetailsDialog, printer_model_stack);
   gtk_widget_class_bind_template_child (widget_class, PpDetailsDialog, search_for_drivers_button);
+  gtk_widget_class_bind_template_child (widget_class, PpDetailsDialog, driver_buttons);
 
-  gtk_widget_class_bind_template_callback (widget_class, printer_name_edit_cb);
   gtk_widget_class_bind_template_callback (widget_class, printer_name_changed);
-  gtk_widget_class_bind_template_callback (widget_class, printer_location_edit_cb);
   gtk_widget_class_bind_template_callback (widget_class, search_for_drivers);
   gtk_widget_class_bind_template_callback (widget_class, select_ppd_in_dialog);
   gtk_widget_class_bind_template_callback (widget_class, select_ppd_manually);
+  gtk_widget_class_bind_template_callback (widget_class, pp_details_dialog_response_cb);
 }
 
 PpDetailsDialog *
@@ -399,7 +413,7 @@ pp_details_dialog_new (GtkWindow            *parent,
 
   /* Translators: This is the title of the dialog. %s is the printer name. */
   title = g_strdup_printf (_("%s Details"), printer_name);
-  gtk_window_set_title (GTK_WINDOW (self), title);
+  gtk_label_set_label (self->dialog_title, title);
 
   printer_url = g_strdup_printf ("<a href=\"http://%s:%d\">%s</a>", printer_address, ippPort (), printer_address);
   gtk_label_set_markup (GTK_LABEL (self->printer_address_label), printer_url);
@@ -409,7 +423,7 @@ pp_details_dialog_new (GtkWindow            *parent,
   gtk_entry_set_text (GTK_ENTRY (self->printer_location_entry), printer_location);
   gtk_label_set_text (GTK_LABEL (self->printer_model_label), printer_make_and_model);
 
-  gtk_widget_set_sensitive (gtk_dialog_get_content_area (GTK_DIALOG (self)), sensitive);
+  update_sensitivity (self, sensitive);
 
   return self;
 }
